use std::fmt::Display;

use serde::{Serialize, Deserialize};

#[derive(Serialize, Deserialize)]
pub struct Card {
    pub suit: Suit,
    pub value: u8,
}

#[derive(Clone, Copy, Serialize, Deserialize)]
pub enum Suit {
    Hearts,
    Diamonds,
    Clubs,
    Spades,
}

impl Card {
    pub fn new(suit: Suit, value: u8) -> Card {
        Card {
            suit,
            value,
        }
    }

    pub fn get_suit(self) -> Suit {
        self.suit
    }

    pub fn get_value(self) -> u8 {
        self.value
    }
}

impl Display for Suit {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        let suit = match self {
            Suit::Diamonds => String::from("♦"),
            Suit::Hearts => String::from("♥"),
            Suit::Spades => String::from("♠"),
            Suit::Clubs => String::from("♣"),
        };

        write!(f, "{}", suit)
    }
}

impl Display for Card {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "[{}{}]", self.value, self.suit)
    }
}
