use anyhow::Result;
use tokio::net::{TcpListener, TcpStream};
use tokio::io::{AsyncBufReadExt, AsyncReadExt, AsyncWriteExt, BufReader};
use tokio::sync::broadcast;

mod card;
mod deck;
mod player;
mod lobby;

#[tokio::main]
async fn main() -> Result<(), anyhow::Error> {
    let address = "127.0.0.1:8080".to_string();
    let listener = TcpListener::bind(&address).await?;

    let (tx, rx) = broadcast::channel::<String>(10);

    while let Ok((mut stream,_)) = listener.accept().await {
        let tx = tx.clone();
        let mut rx = rx.resubscribe();

        tokio::spawn(async move {
            let (read, mut write) = stream.split();
            let mut reader = BufReader::new(read);
            let mut line = String::new();

            loop {
                tokio::select! {
                    bytes_read = reader.read_line(&mut line) => {
                        if bytes_read.unwrap() == 0 {
                            println!("connection closed");
                            return;
                        }
                        tx.send(line.clone()).unwrap();
                    },
                    msg = rx.recv() => {
                        write.write_all(format!("> {}", msg.unwrap()).as_bytes()).await.expect("failed to write data");
                        line.clear();
                    }
                }
            }
        });
    }

    Ok(())
}

async fn spawn(mut stream: TcpStream) -> Result<(), anyhow::Error> {
    todo!()
}
